#pragma once        // MSVC
#ifndef PGCCLIENT_H // QT
//! \cond
#define PGCCLIENT_H
//! \endcond

//! \file
//! Payment Gateway Client Header
//! \n Strings are all in UTF-8 encoding
//! \n Schemes and supported actions
//! \n   WSS   (Async): Request, Confirm, Send
//! \n   WSS   (Sync):  Request
//! \n   HTTPS (Sync):  Request

////////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
extern "C" {
#endif

//! \name ConstsSec
//! Constants section
//!@{

//! Version
#define PGC_API_VERSION "1.68.170829"

////////////////////////////////////////////////////////////////////////////////

//! Error codes
enum PgcErrs
{
    PGC_ERR_NONE        = 0,    //!< No error
    PGC_ERR_NONE_ASYNC  = -1,   //!< No error, async
    PGC_ERR_NONE_FIRST  = -11,  //!< No error, first
    PGC_ERR_NONE_LAST   = -12,  //!< No error, last
    PGC_ERR_NONE_MARKER = -100, //!< No error, marker
    PGC_ERR_GENERAL      = -101, //!< General error
    PGC_ERR_NOTSUPPORTED = -102, //!< Not supported
    PGC_ERR_PARAM_INVALID      = -111, //!< Parameter invalid
    PGC_ERR_SVC_NOTFOUND       = -121, //!< Service not found
    PGC_ERR_SVC_REGISTERFAILED = -122, //!< Register-service failed
    PGC_ERR_CONN_NOTFOUND            = -201, //!< Connection not found
    PGC_ERR_CONN_INITFAILED          = -202, //!< Connection-initialize failed
    PGC_ERR_CONN_OPTNOTFOUND         = -203, //!< Connection option not found
    PGC_ERR_CONN_ALREADYCONNECTED    = -211, //!< Connection already connected
    PGC_ERR_CONN_ALREADYDISCONNECTED = -212, //!< Connection already disconnected
    PGC_ERR_CONN_NOTCONNECTED        = -213, //!< Connection not connected
    PGC_ERR_CONN_CBKNOTREGISTERED    = -221, //!< Connection callback not registered
    PGC_ERR_CONN_ALREADYLISTENING    = -231, //!< Connection already listening
    PGC_ERR_CONN_NOTLISTENING        = -232, //!< Connection not listening
    PGC_ERR_SESS_NOTFOUND         = -301, //!< Session not found
    PGC_ERR_SESS_INITFAILED       = -302, //!< Session-initialize failed
    PGC_ERR_SESS_MSGCNTINVALID    = -311, //!< Session message count invalid
    PGC_ERR_SESS_MSGCNTALREADYSET = -312, //!< Session message count already set
    PGC_ERR_SESS_NOTSOURCE        = -321, //!< Session not in source state
    PGC_ERR_MSG_NOTFOUND      = -401, //!< Message not found
    PGC_ERR_MSG_IDXOUTOFBOUND = -402, //!< Message index out of bound
    PGC_ERR_MSG_FIELDNOTFOUND = -403, //!< Message field not found
    PGC_ERR_MSG_OBJINVALID    = -404, //!< Message object invalid
    PGC_ERR_MSG_SIZEINVALID   = -405, //!< Message size invalid
    PGC_ERR_MSG_KEYINVALID    = -406, //!< Message key invalid
    PGC_ERR_MSG_WRITEFAILED   = -411, //!< Write-message failed
    PGC_ERR_NET_URLINVALID       = -501, //!< Network URL invalid
    PGC_ERR_NET_REFINVALID       = -502, //!< Network reference invalid
    PGC_ERR_NET_INITFAILED       = -503, //!< Initialize-network failed
    PGC_ERR_NET_INFONOTFOUND     = -504, //!< Network info not found
    PGC_ERR_NET_STSNOTFOUND      = -505, //!< Network status/statistics not found
    PGC_ERR_NET_CONNECTFAILED    = -511, //!< Network-connect failed
    PGC_ERR_NET_HANDSHAKEFAILED  = -512, //!< Network-handshake failed
    PGC_ERR_NET_DISCONNECTFAILED = -513, //!< Network-disconnect failed
    PGC_ERR_NET_SENDFAILED       = -521, //!< Network-send failed
    PGC_ERR_NET_RECVFAILED       = -522, //!< Network-receive failed
    PGC_ERR_NET_FLUSHFAILED      = -523, //!< Network-flush failed
    PGC_ERR_NET_PERFORMFAILED    = -524, //!< Network-perform failed
    PGC_ERR_NET_TIMEDOUT         = -531, //!< Network timed out
    PGC_ERR_NET_LISTENFAILED     = -541, //!< Network-listen failed
    PGC_ERR_NET_UNLISTENFAILED   = -542, //!< Network-unlisten failed
    PGC_ERR_NET_CLOSED           = -551, //!< Network-closed
    PGC_ERR_EXT_IDINVALID  = -10101, //!< Ext: ID invalid
    PGC_ERR_EXT_KEYINVALID = -10102, //!< Ext: Key invalid
    PGC_ERR_EXT_TAGINVALID = -10103, //!< Ext: Tag invalid
    PGC_ERR_EXT_PWDINVALID = -10111, //!< Ext: password invalid
    PGC_ERR_EXT_METHODINVALID = -10121, //!< Ext: method invalid
    PGC_ERR_EXT_IDNOTFOUND    = -10201, //!< Ext: ID not found
    PGC_ERR_EXT_KEYNOTFOUND   = -10202, //!< Ext: Key not found
    PGC_ERR_EXT_TAGNOTFOUND   = -10203, //!< Ext: Tag not found
    PGC_ERR_EXT_IDXOUTOFBOUND = -10304, //!< Ext: Index out of bound
    PGC_ERR_EXT_PWDNOTMATCHED = -10411, //!< Ext: Password not matched
    PGC_ERR_EXT_ALREADYDONE   = -10701, //!< Ext: Already done
    PGC_ERR_END               = -99999  //!< Err: End
};


//! Data types
enum PgcTypes
{
    PGC_TYPE_NULL = 0,   //!< Null; Unknown
    PGC_TYPE_STR  = 101, //!< String
    PGC_TYPE_NUM  = 102, //!< Number
    PGC_TYPE_BOOL = 103, //!< Boolean
    PGC_TYPE_DATE = 111, //!< Date/time
    PGC_TYPE_INT  = 121, //!< Integer
    PGC_TYPE_AMT  = 122, //!< Amount
    PGC_TYPE_OBJ  = 201, //!< Object
    PGC_TYPE_ARR  = 202  //!< Array
};


//! Timeout Indicators (used in the nTimeout field in NewConn function)
enum PgcTimeoInds
{
    PGC_TIMEO_DEF    = 0,    //!< Default
    PGC_TIMEO_API    = -101, //!< API: 5 secs
    PGC_TIMEO_API_LL = -102, //!< API with low latency: 2 secs
    PGC_TIMEO_TXN    = -111, //!< Transaction: 10 secs
    PGC_TIMEO_TXN_UI = -113, //!< Transaction with UI: 2 mins
    PGC_TIMEO_BATCH  = -121, //!< Batch: 5 mins
    PGC_TIMEO_MSG    = -131, //!< Message: 15 secs
    PGC_TIMEO_MSG_LL = -132, //!< Message with low latency: 5 secs
    PGC_TIMEO_FILE   = -141  //!< File: 2 hrs
};


//! Message operation targets (used in the nMsgCnt field in NewSess function)
enum PgcMsgOpTgts
{
    PGC_MSGOP_REGULAR = 1,   //!< Regular message
    PGC_MSGOP_SESSDEF = -101 //!< Session default message
};

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name MacrosSec
//! Macros section
//!@{

//! Check if the error code contains an error
//! \param[in] nErr Error code
//! \return True if it contains an error
#define PGC_HAS_ERR(nErr)   ((nErr) < PGC_ERR_NONE_MARKER)


//! Check if the error code is async
//! \param[in] nErr Error code
//! \return True if it is async code
#define PGC_IS_ASYNC(nErr)  ((nErr) == PGC_ERR_NONE_ASYNC)


//! Check if the connection value is valid
//! \param[in] nConn Connection
//! \return True if the connection is valid
#define PGC_ISVALID_CONN(nConn) ((nConn) > 0)


//! Check if the session value is valid
//! \param[in] nSess Session
//! \return True if the session is valid
#define PGC_ISVALID_SESS(nSess) ((nSess) > 0)

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncDecoSec
//! Function decorators section
//!@{

//! Export function decorator
#if defined(_WIN32) || defined(_WIN64)
#define PGC_DECL_EXPORT __declspec(dllexport)
#else
#define PGC_DECL_EXPORT __attribute__ ((visibility("default")))
#endif // WINDOWS / LINUX

//! Import function decorator
#if defined(_WIN32) || defined(_WIN64)
#define PGC_DECL_IMPORT __declspec(dllimport)
#else
#define PGC_DECL_IMPORT
#endif // WINDOWS / LINUX

//! API export function decorator
#ifdef PGC_BUILD_LIB
#  define PGC_API_EXPORT PGC_DECL_EXPORT
#else
#  define PGC_API_EXPORT PGC_DECL_IMPORT
#endif // PGC_BUILD_LIB


//! API entry function decorator
#if defined(_WIN32) || defined(_WIN64)
#define PGC_API_ENTRY __stdcall
#else
#define PGC_API_ENTRY
#endif // WINDOWS / LINUX

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncInitSec
//! Init functions section
//!@{

//! Initialize
//! It has to be called in the main thread
//! \param[in] sMduName Module name
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcInit(const char * sMduName);


//! Uninitialize
//! \param[in] sMduName Module name
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcUninit(const char * sMduName);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncInfoSec
//! Info functions section
//!@{

//! Get version
//! \return Version string
PGC_API_EXPORT const char * PGC_API_ENTRY PgcGetVer();


//! Get error tag
//! \param[in] nErrCode Error code
//! \param[out] sErrTag Error tag; The size must be equal to or greater than 256
PGC_API_EXPORT void PGC_API_ENTRY PgcGetErrTag(int nErrCode, char * sErrTag);


//! Write to log at info level
//! \param[in] sFmt Format
//! \param[in] sArg1 Argument 1 to be logged
//! \param[in] sArg2 Argument 2 to be logged
//! \param[in] sArg3 Argument 3 to be logged
PGC_API_EXPORT void PGC_API_ENTRY PgcLogInfo(const char * sFmt, const char * sArg1, const char * sArg2, const char * sArg3);


//! Write to log at error level
//! \param[in] sFmt Format
//! \param[in] sArg1 Argument 1 to be logged
//! \param[in] sArg2 Argument 2 to be logged
//! \param[in] sArg3 Argument 3 to be logged
PGC_API_EXPORT void PGC_API_ENTRY PgcLogErr(const char * sFmt, const char * sArg1, const char * sArg2, const char * sArg3);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncConnSec
//! Connection functions section
//!@{

//! Create new connection
//! \param[in] sUrl URL to connect to; Accepted forms: "wss://host:port/path", "https://host:port/path", or "$Service"; if empty, get from: cfg file in curr dir, cfg file in def dir
//! \param[in] sUName Name that can uniquely identify the client; if empty, get from: env var, cfg file in curr dir, cfg file in def dir, random name
//! \param[in] sPwd Password to connect to the server; if empty, get from: cfg file in curr dir, cfg file in def dir
//! \param[in] nTimeout Timeout indicator or actual timeout in milliseconds; if zero, get from: cfg file in curr dir, cfg file in def dir
//! \param[in] sMduName Module name; if empty, get from: random name
//! \param[in] sCfgPath Configuration directory or file; if file path, use it; if dir path, locate cfg file; if empty, use curr dir; if not in curr dir, use def dir; any file matching prefix and extension in the cfg dir; if file name has matching UName, choose this
//! \return Connection or Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcNewConn(const char * sUrl, const char * sUName, const char * sPwd, int nTimeout, const char * sMduName, const char * sCfgPath);


//! Close connection
//! \param[in] nConn Connection
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcFreeConn(int nConn);


//! Is connection synchronous mode (non-async mode)
//! \param[in] nConn Connection
//! \return True if it is synchronous mode
PGC_API_EXPORT bool PGC_API_ENTRY PgcIsSyncMode(int nConn);


//! Set connection option
//! \param[in] nConn Connection
//! \param[in] sKey Key for the option
//! \param[in] sVal Value for the option
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetConnOpt(int nConn, const char * sKey, const char * sVal);


//! Get connection status / stat
//! \param[in] nConn Connection
//! \param[in] sKey Key of the status / stat
//! \param[out] sVal Value buffer for the status / stat
//! \param[in] nSize Size of the value buffer
//! \return Error code, or if Val is NULL, then size is returned
PGC_API_EXPORT int PGC_API_ENTRY PgcGetConnSts(int nConn, const char * sKey, char * sVal, int nSize);


//! Set connection property
//! \param[in] nConn Connection
//! \param[in] sSec Section of the property
//! \param[in] sKey Key of the property
//! \param[in] sVal Value buffer for the property
//! \return Error code, or if Val is NULL, then size is returned
PGC_API_EXPORT int PGC_API_ENTRY PgcSetConnProp(int nConn, const char * sSec, const char * sKey, const char * sVal);


//! Get connection property
//! \param[in] nConn Connection
//! \param[in] sSec Section of the property
//! \param[in] sKey Key of the property
//! \param[in] sDefVal Default value of the property
//! \param[out] sVal Value buffer for the property
//! \param[in] nSize Size of the value buffer
//! \return Error code, or if Val is NULL, then size is returned
PGC_API_EXPORT int PGC_API_ENTRY PgcGetConnProp(int nConn, const char * sSec, const char * sKey, const char * sDefVal, char * sVal, int nSize);


//! Connect
//! \param[in] nConn Connection
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcConnect(int nConn);


//! Disconnect
//! \param[in] nConn Connection
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcDisconnect(int nConn);


//! Is connected
//! \param[in] nConn Connection
//! \return True if it is connected
PGC_API_EXPORT bool PGC_API_ENTRY PgcIsConnected(int nConn);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncSessSec
//! Session functions section
//!@{

//! Create new session
//! \param[in] nConn Connection
//! \param[in] nMsgCnt Message operation target or the actual message count
//! \return Session or Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcNewSess(int nConn, int nMsgCnt);


//! Close session
//! \param[in] nSess Session
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcFreeSess(int nSess);


//! Is session a receiving party
//! \param[in] nSess Session
//! \return True if it is received
PGC_API_EXPORT bool PGC_API_ENTRY PgcIsRecvEnd(int nSess);


//! Set session option
//! \param[in] nSess Session
//! \param[in] sKey Key for the option
//! \param[in] sVal Value for the option
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetSessOpt(int nSess, const char * sKey, const char * sVal);


//! Get session status / stat
//! \param[in] nSess Session
//! \param[in] sKey Key of the status / stat
//! \param[out] sVal Value buffer for the status / stat
//! \param[in] nSize Size of the value buffer
//! \return Error code, or if Val is NULL, then size is returned
PGC_API_EXPORT int PGC_API_ENTRY PgcGetSessSts(int nSess, const char * sKey, char * sVal, int nSize);


//! Set connection to session
//! \param[in] nSess Session
//! \param[in] nConn Connection
//! \return Connection or Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetConn(int nSess, int nConn);


//! Get connection from session
//! \param[in] nSess Session
//! \return Connection or Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcGetConn(int nSess);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncMsgBlockSec
//! Message block functions section
//!@{

//! Get message count
//! \param[in] nSess Session
//! \return Message count
PGC_API_EXPORT int PGC_API_ENTRY PgcGetMsgCnt(int nSess);


// Set message data
//! \param[in] nSess Session
//! \param[in] bIsRaw if the data is raw data (not JSON)
//! \param[in] sVal Value of the data
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetMsgData(int nSess, bool bIsRaw, const char * sVal);


// Get message data
//! \param[in] nSess Session
//! \param[in] bIsRaw if the data is raw data (not JSON)
//! \param[out] sVal Value of the data
//! \param[in] nSize Size of the sVal buffer
//! \return Error code; or if sVal is NULL or nSize is 0, then size is returned
PGC_API_EXPORT int PGC_API_ENTRY PgcGetMsgData(int nSess, bool bIsRaw, char * sVal, int nSize);


// Copy message data
//! \param[in] nSess Session
//! \param[in] nSrcSess Source session
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcCopyMsgData(int nSess, int nSrcSess);


//! Set message parameter
//! \param[in] nSess Session
//! \param[in] sKey Key of the data
//! \param[in] sVal Value of the data
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetMsgPrm(int nSess, const char * sKey, const char * sVal);


//! Get message parameter
//! \param[in] nSess Session
//! \param[in] sKey Key of the data
//! \param[out] sVal Value of the data
//! \param[in] nSize Size of the sVal buffer
//! \return Error code; or if sVal is NULL or nSize is 0, then size is returned
PGC_API_EXPORT int PGC_API_ENTRY PgcGetMsgPrm(int nSess, const char * sKey, char * sVal, int nSize);


// Is message a raw message (not JSON)
//! \param[in] nSess Session
//! \return True if the message is a raw message (not JSON)
PGC_API_EXPORT bool PGC_API_ENTRY PgcIsRawMsg(int nSess);


//! Create new message for confirm or reply
//! The old message is backed up and can be accessed using negative message indexes
//! \param[in] nSess Session
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcNewMsg(int nSess);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncMsgSec
//! Message functions section
//!@{

//! Is key set in message
//! \param[in] nSess Session
//! \param[in] nMsg Message index: starts with 0 for current messages; use negative numbers for the messages that were just backed up
//! \param[in] sKey Key of the data; Accepted forms: Pointer style keys: /RootObj/SubObj/ArrIdx/Key
//! \return True if the data with the key is set in the message
PGC_API_EXPORT bool PGC_API_ENTRY PgcIsSet(int nSess, int nMsg, const char * sKey);


//! Unset key in message
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \return Error code
PGC_API_EXPORT int  PGC_API_ENTRY PgcUnset(int nSess, int nMsg, const char * sKey);


//! Is value empty in message
//! \param[in] nSess Session
//! \param[in] nMsg Message index: starts with 0 for current messages; use negative numbers for the messages that were just backed up
//! \param[in] sKey Key of the data; Accepted forms: Pointer style keys: /RootObj/SubObj/ArrIdx/Key
//! \return True if the data with the key is empty or not set in the message
PGC_API_EXPORT bool PGC_API_ENTRY PgcIsEmpty(int nSess, int nMsg, const char * sKey);


//! Get type of value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \return Data type
PGC_API_EXPORT int PGC_API_ENTRY PgcGetType(int nSess, int nMsg, const char * sKey);


//! Set string value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \param[in] sVal Value of the data
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetStr(int nSess, int nMsg, const char * sKey, const char * sVal);


//! Get string value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \param[out] sVal Value of the data
//! \param[in] nSize Size of the sVal buffer
//! \return Error code; or if sVal is NULL or nSize is 0, then size is returned
PGC_API_EXPORT int PGC_API_ENTRY PgcGetStr(int nSess, int nMsg, const char * sKey, char * sVal, int nSize);


//! Set amount value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \param[in] nVal Value of the data; 2 decimal places assumed
//! \return Error code
PGC_API_EXPORT int  PGC_API_ENTRY PgcSetAmt(int nSess, int nMsg, const char * sKey, long nVal);


//! Get amount value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \return Amount value
PGC_API_EXPORT long PGC_API_ENTRY PgcGetAmt(int nSess, int nMsg, const char * sKey);


//! Set integer value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \param[in] nVal Value of the data
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetInt(int nSess, int nMsg, const char * sKey, int nVal);


//! Get integer value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \return Integer value
PGC_API_EXPORT int PGC_API_ENTRY PgcGetInt(int nSess, int nMsg, const char * sKey);


//! Set number value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \param[in] nVal Value of the data
//! \return Error code
PGC_API_EXPORT int    PGC_API_ENTRY PgcSetNum(int nSess, int nMsg, const char * sKey, double nVal);


//! Get number value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \return Number value
PGC_API_EXPORT double PGC_API_ENTRY PgcGetNum(int nSess, int nMsg, const char * sKey);


//! Set boolean value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \param[in] bVal Value of the data
//! \return Error code
PGC_API_EXPORT int  PGC_API_ENTRY PgcSetBool(int nSess, int nMsg, const char * sKey, bool bVal);


//! Get boolean value
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \return Boolean value
PGC_API_EXPORT bool PGC_API_ENTRY PgcGetBool(int nSess, int nMsg, const char * sKey);


//! Set date/time value defined as seconds since 00:00, Jan 1, 1970 UTC
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \param[in] dVal Value of the data; floating-point time_t
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetDate(int nSess, int nMsg, const char * sKey, double dVal);


//! Get date/time value defined as seconds since 00:00, Jan 1, 1970 UTC
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \return Date value
PGC_API_EXPORT double PGC_API_ENTRY PgcGetDate(int nSess, int nMsg, const char * sKey);


//! Set child message in a session; a message (encoded) within the message
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \param[in] nChildMsgSess Session that contains the child message
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetChildMsg(int nSess, int nMsg, const char * sKey, int nChildMsgSess);


//! Get child message in a session; a message (encoded) within the message
//! \param[in] nSess Session
//! \param[in] nMsg Message index
//! \param[in] sKey Key of the data
//! \return Integer value of the session that contains the child message
PGC_API_EXPORT int PGC_API_ENTRY PgcGetChildMsg(int nSess, int nMsg, const char * sKey);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncReqSec
//! Request functions section
//!@{

//! Request
//! \param[in] nSess Session
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcRequest(int nSess);


//! Send message (from client, either initiate or respond)
//! \param[in] nSess Session
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSend(int nSess);


//! Perform task (from client)
//! \param[in] nSess Session
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcPerform(int nSess);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncCbkSec
//! Callback functions section
//!@{

//! Event callback function
/*! Example: int PGC_API_ENTRY PgcCallback(void * pRef, int nConn, int nSess); */
//! \param PGC_EVTCBK_FUNC Event callback function
// ! \param[in] pRef Reference pointer
// ! \param[in] nConn Connection
// ! \param[in] nSess Session
//! \return Error code
typedef int (PGC_API_ENTRY PGC_EVTCBK_FUNC) (void * pRef, int nConn, int nSess);


//! Set callback
/*! To manually check the messages, make pfnEvtCbk and pEvtCbkRef NULL */
//! \param[in] nConn Connection
//! \param[in] pfnEvtCbk Event callback function
//! \param[in] pEvtCbkRef Reference pointer when calling back
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcSetCbk(int nConn, PGC_EVTCBK_FUNC * pfnEvtCbk, void * pEvtCbkRef);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncHeedSec
//! Heed functions section
//!@{

//! Heed (only valid for WSS)
//! \param[in] nConn Connection
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcHeed(int nConn);


//! Unheed
//! \param[in] nConn Connection
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcUnheed(int nConn);


//! Is heeding
//! \param[in] nConn Connection
//! \return True if it is heeding
PGC_API_EXPORT bool PGC_API_ENTRY PgcIsHeeding(int nConn);


//! Poll
//! \param[in] nConn Connection
//! \param[in] nTimeout Timeout waiting for the message; 0 means no wait.
//! \return Session if available, otherwise 0 or error code
PGC_API_EXPORT int PGC_API_ENTRY PgcPoll(int nConn, int nTimeout);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncCfmSec
//! Confirm functions section
//!@{

//! Confirm
//! \param[in] nSess Session
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcConfirm(int nSess);

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name FuncTxnSec
//! Transaction functions section
//!@{

//! Do transaction
//! \param[in] sUrl URL to connect to; Accepted forms: "wss://host:port/", "https://host:port/", or "$Service"
//! \param[in] sReq Request string in JSON format
//! \param[out] sRsp Response string in JSON format
//! \param[in] nRspSize Response buffer size
//! \return Error code
PGC_API_EXPORT int PGC_API_ENTRY PgcDoTxn(const char * sUrl, const char * sReq, char * sRsp, int nRspSize);

//!@}

#ifdef __cplusplus
} // extern "C"
#endif

#endif // PGCCLIENT_H
