public class PgcKeys
{
//! \cond
    //! \endcond

    //! \file
    //! Payment Gateway Client Keys
    //! \n Strings are all in UTF-8 encoding

    ////////////////////////////////////////////////////////////////////////////////

    //! \name KeySepSec
    //! key separator section
    //!@{

    public const string PGC_KEY_SEP_CELL = "-"; // key cell separator (lower than an item); (the result contains one word: up-to-date)
    public const string PGC_KEY_SEP_ITEM = "_"; // key item separator; (the result contains two words, but one term: new_york)
    public const string PGC_KEY_SEP_GRP  = "~"; // key group separator (higher than an item); the result contains two separate terms, but may limit the scope: sales~florida)

    public const string PGC_KEY_SEP_JSON = "/"; // JSON key separator
    public const string PGC_KEY_SEP_PROP = "."; // property key separator

    //!@}


    //! \name KeyEnvSec
    //! Environment variable keys section
    //!@{

    public const string PGC_ENVKEY_UNAME = "PGC_UNAME"; //!< Environment variable: Unique name

    //!@}


    //! \name KeyGenSec
    //! General keys section
    //!@{

    //! General
    public const string PGC_GENVAL_EMPTY = "";  //!< Empty
    public const string PGC_GENVAL_TRUE  = "Y"; //!< True, Yes
    public const string PGC_GENVAL_FALSE = "N"; //!< False, No

    //!@}


    //! \name KeyPropSec
    //! Property keys section
    //!@{

    //! Connection Properties
    public const string PGC_CONNPROPSEC_CONNPRM = "Conn-Prm"; //!< Connection parameter

    public const string PGC_CONNPROPKEY_URL     = "Url";      //!< URL
    public const string PGC_CONNPROPKEY_UNAME   = "UName";    //!< Unique name
    public const string PGC_CONNPROPKEY_PWD     = "Pwd";      //!< Password
    public const string PGC_CONNPROPKEY_TIMEOUT = "Timeout";  //!< Timeout

    public const string PGC_CONNPROPSEC_SESSDEF = "Sess-Def"; //!< Session default

    //!@}


    //! \name KeyOptSec
    //! Option keys section
    //!@{

    //! Connection Options
    public const string PGC_CONNOPT_PREFIX  = "xConnOpt_"; //!< Connection option prefix

    public const string PGC_CONNOPTKEY_TIMEO        = "xConnOpt_Timeout"; //!< Int: Time out
    public const string PGC_CONNOPTVAL_TIMEO_API    = "5000";   //!< API: 5 secs
    public const string PGC_CONNOPTVAL_TIMEO_API_LL = "2000";   //!< API with low latency: 2 secs
    public const string PGC_CONNOPTVAL_TIMEO_TXN    = "10000";  //!< Transaction: 10 secs
    public const string PGC_CONNOPTVAL_TIMEO_TXN_UI = "120000"; //!< Transaction with UI: 120 secs
    public const string PGC_CONNOPTVAL_TIMEO_BATCH  = "300000"; //!< Batch: 300 secs
    public const string PGC_CONNOPTVAL_TIMEO_MSG    = "15000";  //!< Message: 15 secs
    public const string PGC_CONNOPTVAL_TIMEO_MSG_LL = "5000";   //!< Message with low latency: 5 secs
    public const string PGC_CONNOPTVAL_TIMEO_FILE   = "7200000"; //!< File: 2 hrs

    public const string PGC_CONNOPTKEY_MSGFRAMING      = "xConnOpt_MsgFraming"; //!< Str: Message framing; Def: 101
    public const string PGC_CONNOPTVAL_MSGFRAMING_STX  = "101";                 //!< Str: STX/ETX message framing

    public const string PGC_CONNOPTKEY_USESTRFMT       = "xConnOpt_UseStrFmt"; //!< Bool: Use string format; Def: Y

    public const string PGC_CONNOPTKEY_CLNCERTFILEPATH = "xConnOpt_ClnCertFilePath"; //!< Str: Client certificate file path; Def: pgccrt_cln.pem
    public const string PGC_CONNOPTKEY_SVRCERTFILEPATH = "xConnOpt_SvrCertFilePath"; //!< Str: Server certificate file path; Def: pgccrt_svr.pem
    public const string PGC_CONNOPTKEY_SVRKEYFILEPATH  = "xConnOpt_SvrKeyFilePath";  //!< Str: Server key file path;         Def: pgckey_svr.pem

    public const string PGC_CONNOPTKEY_CIPHERLIST      = "xConnOpt_CipherList";      //!< Str: Cipher list;                  Def: empty (can be set to = "DEFAULT:!SHA:!MD5:!3DES:!RC4")

    public const string PGC_CONNOPTKEY_LASTERRCODE     = "xConnOpt_LastErrCode"; //!< Int: Last error code, set it (reset it)

    public const string PGC_CONNOPTKEY_ISSYNCMODE      = "xConnOpt_IsSyncMode";   //!< Bool: Is synchronous (non-async) mode; Def: N
    public const string PGC_CONNOPTKEY_ISMULTITRANS    = "xConnOpt_IsMultiTrans"; //!< Bool: Is multi-trans mode (not disconnect after each request); Def: Y

    public const string PGC_CONNOPTKEY_WAITBEFOREHANDSHAKE  = "xConnOpt_WaitBeforeHandshake";   //!< Int: Wait before handshake;  Def: 50
    public const string PGC_CONNOPTKEY_WAITBEFOREHANDSHAKE2 = "xConnOpt_WaitBeforeHandshake2";  //!< Int: Wait before handshake2; Def: 50
    public const string PGC_CONNOPTKEY_WAITBEFORERECEIVE    = "xConnOpt_WaitBeforeReceive";     //!< Int: Wait before receive in request; Def: 50

    public const string PGC_CONNOPTKEY_WAITAFTERCONNECT = "xConnOpt_WaitAfterConnect";  //!< Int: Wait after connect; Def: 100
    public const string PGC_CONNOPTKEY_WAITAFTERHEED    = "xConnOpt_WaitAfterHeed";     //!< Int: Wait after heed;    Def: 100
    public const string PGC_CONNOPTKEY_WAITAFTERLISTEN  = "xConnOpt_WaitAfterListen";   //!< Int: Wait after listen;  Def: 100
    public const string PGC_CONNOPTKEY_WAITAFTERPOLL    = "xConnOpt_WaitAfterPoll";     //!< Int: Wait after poll;   Def: 10
    public const string PGC_CONNOPTKEY_WAITAFTERACCEPT  = "xConnOpt_WaitAfterAccept";   //!< Int: Wait after accept; Def: 10

    public const string PGC_CONNOPTKEY_WAITAFTERFAILEDCONNECT = "xConnOpt_WaitAfterFailedConnect";  //!< Int: Wait after failed connect; Def: 1000


    //! Session Options

    //!@}


    //! \name KeyStsSec
    //! Status/Statistics keys section
    //!@{

    //! Connection Status/Statistics
    public const string PGC_CONNSTSKEY_PREFIX       = "xConnSts_";  //!< Connection stat prefix

    public const string PGC_CONNSTSKEY_LASTERRCODE  = "xConnSts_LastErrCode";  //!< Int: Last error code

    public const string PGC_CONNSTSPFX_CAP          = "xConnSts_Cap_";         //!< Prefix for Capability
    public const string PGC_CONNSTSKEY_CAP_SYNCCLN  = "xConnSts_Cap_SyncCln";  //!< Bln: Capability: Supports sync client
    public const string PGC_CONNSTSKEY_CAP_ASYNCCLN = "xConnSts_Cap_AsyncCln"; //!< Bln: Capability: Supports async client
    public const string PGC_CONNSTSKEY_CAP_FILECLN  = "xConnSts_Cap_FileCln";  //!< Bln: Capability: Supports file client
    public const string PGC_CONNSTSKEY_CAP_ASYNCSVR = "xConnSts_Cap_AsyncSvr"; //!< Bln: Capability: Supports async server

    public const string PGC_CONNSTSPFX_TS                    = "xConnSts_TS_";                  //!< Prefix for Timestamp
    public const string PGC_CONNSTSKEY_TS_INIT_BEGIN         = "xConnSts_TS_InitBegin";         //!< Str: Timestamp: Init Begin
    public const string PGC_CONNSTSKEY_TS_INIT_END           = "xConnSts_TS_InitEnd";           //!< Str: Timestamp: Init End
    public const string PGC_CONNSTSKEY_TS_UNINIT_BEGIN       = "xConnSts_TS_UninitBegin";       //!< Str: Timestamp: Uninit Begin
    public const string PGC_CONNSTSKEY_TS_UNINIT_END         = "xConnSts_TS_UninitEnd";         //!< Str: Timestamp: Uninit End
    public const string PGC_CONNSTSKEY_TS_CONNECT_BEGIN      = "xConnSts_TS_ConnectBegin";      //!< Str: Timestamp: Connect Begin
    public const string PGC_CONNSTSKEY_TS_CONNECT_HANDSHAKE  = "xConnSts_TS_ConnectHandshake";  //!< Str: Timestamp: Connect Handshake (TLS)
    public const string PGC_CONNSTSKEY_TS_CONNECT_HANDSHAKE2 = "xConnSts_TS_ConnectHandshake2"; //!< Str: Timestamp: Connect Handshake2 (WSS upgrade)
    public const string PGC_CONNSTSKEY_TS_CONNECT_END        = "xConnSts_TS_ConnectEnd";        //!< Str: Timestamp: Connect End
    public const string PGC_CONNSTSKEY_TS_CONNECT_FAILED     = "xConnSts_TS_ConnectFailed";     //!< Str: Timestamp: Connect Failed
    public const string PGC_CONNSTSKEY_TS_DISCONNECT_BEGIN   = "xConnSts_TS_DisconnectBegin";   //!< Str: Timestamp: Disconnect Begin
    public const string PGC_CONNSTSKEY_TS_DISCONNECT_END     = "xConnSts_TS_DisconnectEnd";     //!< Str: Timestamp: Disconnect End
    public const string PGC_CONNSTSKEY_TS_REQUEST_BEGIN      = "xConnSts_TS_RequestBegin";      //!< Str: Timestamp: Request Begin
    public const string PGC_CONNSTSKEY_TS_REQUEST_RECV       = "xConnSts_TS_RequestRecv";       //!< Str: Timestamp: Request Recv
    public const string PGC_CONNSTSKEY_TS_REQUEST_END        = "xConnSts_TS_RequestEnd";        //!< Str: Timestamp: Request End
    public const string PGC_CONNSTSKEY_TS_SEND_BEGIN         = "xConnSts_TS_SendBegin";         //!< Str: Timestamp: Send Begin
    public const string PGC_CONNSTSKEY_TS_SEND_END           = "xConnSts_TS_SendEnd";           //!< Str: Timestamp: Send End
    public const string PGC_CONNSTSKEY_TS_CONFIRM_BEGIN      = "xConnSts_TS_ConfirmBegin";      //!< Str: Timestamp: Confirm Begin
    public const string PGC_CONNSTSKEY_TS_CONFIRM_END        = "xConnSts_TS_ConfirmEnd";        //!< Str: Timestamp: Confirm End
    public const string PGC_CONNSTSKEY_TS_REPLY_BEGIN        = "xConnSts_TS_ReplyBegin";        //!< Str: Timestamp: Reply Begin
    public const string PGC_CONNSTSKEY_TS_REPLY_END          = "xConnSts_TS_ReplyEnd";          //!< Str: Timestamp: Reply End
    public const string PGC_CONNSTSKEY_TS_NOTIFY_BEGIN       = "xConnSts_TS_NotifyBegin";       //!< Str: Timestamp: Notify Begin
    public const string PGC_CONNSTSKEY_TS_NOTIFY_END         = "xConnSts_TS_NotifyEnd";         //!< Str: Timestamp: Notify End
    public const string PGC_CONNSTSKEY_TS_LISTEN_BEGIN       = "xConnSts_TS_ListenBegin";       //!< Str: Timestamp: Listen Begin
    public const string PGC_CONNSTSKEY_TS_LISTEN_END         = "xConnSts_TS_ListenEnd";         //!< Str: Timestamp: Listen End
    public const string PGC_CONNSTSKEY_TS_UNLISTEN_BEGIN     = "xConnSts_TS_UnlistenBegin";     //!< Str: Timestamp: Unlisten Begin
    public const string PGC_CONNSTSKEY_TS_UNLISTEN_END       = "xConnSts_TS_UnlistenEnd";       //!< Str: Timestamp: Unlisten End
    public const string PGC_CONNSTSKEY_TS_HEED_BEGIN         = "xConnSts_TS_HeedBegin";         //!< Str: Timestamp: Heed Begin
    public const string PGC_CONNSTSKEY_TS_HEED_END           = "xConnSts_TS_HeedEnd";           //!< Str: Timestamp: Heed End
    public const string PGC_CONNSTSKEY_TS_UNHEED_BEGIN       = "xConnSts_TS_UnheedBegin";       //!< Str: Timestamp: Unheed Begin
    public const string PGC_CONNSTSKEY_TS_UNHEED_END         = "xConnSts_TS_UnheedEnd";         //!< Str: Timestamp: Unheed End
    public const string PGC_CONNSTSKEY_TS_PERFORM_BEGIN      = "xConnSts_TS_PerformBegin";      //!< Str: Timestamp: Perform Begin
    public const string PGC_CONNSTSKEY_TS_PERFORM_END        = "xConnSts_TS_PerformEnd";        //!< Str: Timestamp: Perform End
    public const string PGC_CONNSTSKEY_TS_POLL_BEGIN         = "xConnSts_TS_PollBegin";         //!< Str: Timestamp: Poll Begin
    public const string PGC_CONNSTSKEY_TS_POLL_END           = "xConnSts_TS_PollEnd";           //!< Str: Timestamp: Poll End
    public const string PGC_CONNSTSKEY_TS_ACCEPT_BEGIN       = "xConnSts_TS_AcceptBegin";       //!< Str: Timestamp: Accept Begin
    public const string PGC_CONNSTSKEY_TS_ACCEPT_END         = "xConnSts_TS_AcceptEnd";         //!< Str: Timestamp: Accept End

    public const string PGC_CONNSTSKEY_PEERLIST = "xConnSts_PeerList"; //!< List of connected peers


    //! Session Status/Statistics

    //!@}


    //! \name KeyInfoSec
    //! Info keys section
    //!@{

    //! Peer Info (Other party end point)
    public const string PGC_PEERINFOKEY_URL      = "xPeerInfo_URL";      //!< URL
    public const string PGC_PEERINFOKEY_ADDR     = "xPeerInfo_Addr";     //!< Address
    public const string PGC_PEERINFOKEY_PORT     = "xPeerInfo_Port";     //!< Port
    public const string PGC_PEERINFOKEY_ADDRPORT = "xPeerInfo_AddrPort"; //!< Address + Port
    public const string PGC_PEERINFOKEY_CONNNUM  = "xPeerInfo_ConnNum";  //!< Connection number
    public const string PGC_PEERINFOKEY_PEERLIST = "xPeerInfo_PeerList"; //!< List of connected peers

    //!@}


    //! \name KeyHostSec
    //! Host keys section
    //!@{

    //! Hosts: special host names have brackets around
    public const string PGC_HOST_ANY   = "[any]";     //!< broadcast
    public const string PGC_HOST_SVR   = "[svr]";     //!< server
    public const string PGC_HOST_LOCAL = "localhost"; //!< local host
    public const string PGC_IP_LOCAL   = "127.0.0.1"; //!< local host IP
    public const string PGC_IP_BCAST   = "255.255.255.255"; //!< broadcast IP
    public const string PGC_IP_ALL     = "0.0.0.0";   //!< server IP

    //! Lists: special list elements
    public const string PGC_LIST_HOSTPORT_ALL = "(all)"; //!< all host:port pairs
    public const string PGC_LIST_ADDRPORT_ALL = "(all)"; //!< all addr:port pairs
    public const string PGC_LIST_TOPIC_ALL    = "(all)"; //!< all topics

    //!@}


    //! \name KeySvcSec
    //! Service keys section
    //!@{

    //! WSL: WEB Service Locator: service names start with $; web scope service names start with $WEB-
    public const string PGC_SVC_C9PG_PWE      = "$WEB-C9PG-PWE";      //!< Payment Web Engine
    public const string PGC_SVC_C9PG_WBS      = "$WEB-C9PG-WBS";      //!< Web Batch Service
    public const string PGC_SVC_C9PG_NUM      = "$WEB-C9PG-NUM";      //!< Number Service

    public const string PGC_SVC_C9PG_PWE_TEST = "$WEB-C9PG-PWE-TEST"; //!< Payment Web Engine Test
    public const string PGC_SVC_C9PG_WBS_TEST = "$WEB-C9PG-WBS-TEST"; //!< Web Batch Service Test
    public const string PGC_SVC_C9PG_NUM_TEST = "$WEB-C9PG-NUM-TEST"; //!< Number Service Test

    //! LSL: LAN Service Locator: service names start with $; LAN scope service names start with $LAN-
    public const string PGC_SVC_N1CL_TXN = "$LAN-N1CL-TXN"; //!< CreditLine Transaction Service
    public const string PGC_SVC_N1CL_API = "$LAN-N1CL-API"; //!< CreditLine API Service

    //! HSL: HOST Service Locator: service names start with $; host scope service names start with $HOST-
    public const string PGC_SVC_C9PG_PDC = "$HOST-C9PG-PDC"; //!< Payment Device Controller
    public const string PGC_SVC_C9PG_DBS = "$HOST-C9PG-DBS"; //!< Device Batch Service


    // URLs - defaults
    public const string PGC_URL_C9PG_PWE      = "https://link.c9pg.com:5568/restApi";  //!< Payment Web Engine default URL; = "https://link.c9pg.com:5551/hj"
    public const string PGC_URL_C9PG_WBS      = "https://link.c9pg.com:5566/bchApi";   //!< Web Batch Service default URL
    public const string PGC_URL_C9PG_NUM      = "https://link.c9pg.com:16869/numApi";  //!< Number Service default URL

    public const string PGC_URL_C9PG_PWE_TEST = "https://testlink.c9pg.com:5568/restApi";  //!< Payment Web Engine Test default URL; = "https://testlink.c9pg.com:5551/hj"
    public const string PGC_URL_C9PG_WBS_TEST = "https://testlink.c9pg.com:5566/bchApi";   //!< Web Batch Service Test default URL
    public const string PGC_URL_C9PG_NUM_TEST = "https://testlink.c9pg.com:16869/numApi";  //!< Number Service Test default URL

    public const string PGC_URL_C9PG_PDC      = "wss://127.0.0.1:5556/";             //!< Payment Device Controller default URL
    public const string PGC_URL_C9PG_DBS      = "wss://127.0.0.1:5556/";             //!< Device Batch Service default URL

    //!@}


    //! \name KeyMsgPrmSec
    //! Message parameter keys section
    //!@{

    public const string PGC_MSGPRMKEY_DATE    = "xMsgPrm_Date";    //!< Str: date
    public const string PGC_MSGPRMKEY_FROM    = "xMsgPrm_From";    //!< Str: from uname
    public const string PGC_MSGPRMKEY_TO      = "xMsgPrm_To";      //!< Str: list of to unames
    public const string PGC_MSGPRMKEY_SUBJECT = "xMsgPrm_Subject"; //!< Str: subject; topic
    public const string PGC_MSGPRMKEY_THREAD  = "xMsgPrm_Thread";  //!< Str: thread

    public const string PGC_MSGPRMKEY_QOS          = "xMsgPrm_QoS";          //!< Str: quality of service
    public const string PGC_MSGPRMVAL_QOS_LTEQONCE = "xMsgPrm_QoS_LtEqOnce"; //!< Str: QoS: less than or equal to once; at most once; least expensive
    public const string PGC_MSGPRMVAL_QOS_GTEQONCE = "xMsgPrm_QoS_GtEqOnce"; //!< Str: QoS: greater than or equal to once; at least once
    public const string PGC_MSGPRMVAL_QOS_EQONCE   = "xMsgPrm_QoS_EqOnce";   //!< Str: QoS: equal to once; exactly once; most expensive

    public const string PGC_MSGPRMKEY_ISDUP      = "xMsgPrm_IsDup";    //!< Bool Str: is duplicate
    public const string PGC_MSGPRMKEY_TORETAIN   = "xMsgPrm_ToRetain"; //!< Bool Str: to retain

    public const string PGC_MSGPRMKEY_MSGID            = "xMsgPrm_MsgID";        //!< Str: message ID

    public const string PGC_MSGPRMKEY_MSGTYPE           = "xMsgPrm_MsgType";           //!< Str: message type
    public const string PGC_MSGPRMVAL_MSGTYPE_NORMAL    = "xMsgPrm_MsgType_Normal";    //!< Str: Message type: normal
    public const string PGC_MSGPRMVAL_MSGTYPE_CHAT      = "xMsgPrm_MsgType_Chat";      //!< Str: Message type: chat
    public const string PGC_MSGPRMVAL_MSGTYPE_GROUPCHAT = "xMsgPrm_MsgType_GroupChat"; //!< Str: Message type: group chat
    public const string PGC_MSGPRMVAL_MSGTYPE_HEADLINE  = "xMsgPrm_MsgType_Headline";  //!< Str: Message type: headline

    public const string PGC_MSGPRMKEY_PAYLOAD      = "xMsgPrm_Payload";      //!< Str: payload
    public const string PGC_MSGPRMVAL_PAYLOAD_BODY = "xMsgPrm_Payload_Body"; //!< Str: payload: body
    public const string PGC_MSGPRMVAL_PAYLOAD_URL  = "xMsgPrm_Payload_Url";  //!< Str: payload: url

    public const string PGC_MSGPRMKEY_METHOD        = "xMsgPrm_Method";        //!< Str: method
    public const string PGC_MSGPRMVAL_METHOD_POST   = "xMsgPrm_Method_Post";   //!< Str: method: post
    public const string PGC_MSGPRMVAL_METHOD_GET    = "xMsgPrm_Method_Get";    //!< Str: method: get
    //public const string PGC_MSGPRMVAL_METHOD_PUT    = "xMsgPrm_Method_Put";    //!< Str: method: put
    //public const string PGC_MSGPRMVAL_METHOD_DELETE = "xMsgPrm_Method_Delete"; //!< Str: method: delete

    public const string PGC_MSGPRMKEY_SUBPATH       = "xMsgPrm_SubPath";       //!< Str: sub-path (RESTful API)

    public const string PGC_MSGPRMKEY_ACCEPT                = "xMsgPrm_Accept";        //!< Str: accept
    public const string PGC_MSGPRMKEY_CONTENTTYPE           = "xMsgPrm_ContentType";   //!< Str: content type
    public const string PGC_MSGPRMVAL_CONTENTTYPE_JSON      = "application/json";      //!< Str: content type: JSON
    public const string PGC_MSGPRMVAL_CONTENTTYPE_TEXT      = "text/plain";            //!< Str: content type: TEXT

    public const string PGC_MSGPRMKEY_TASK             = "xMsgPrm_Task";             //!< Str: task
    public const string PGC_MSGPRMVAL_TASK_GET         = "xMsgPrm_Task_Get";         //!< Str: Task: get
    public const string PGC_MSGPRMVAL_TASK_PUT         = "xMsgPrm_Task_Put";         //!< Str: Task: put
    public const string PGC_MSGPRMVAL_TASK_SUBSCRIBE   = "xMsgPrm_Task_Subscribe";   //!< Str: Task: subscribe
    public const string PGC_MSGPRMVAL_TASK_UNSUBSCRIBE = "xMsgPrm_Task_Unsubscribe"; //!< Str: Task: unsubscribe

    public const string PGC_MSGPRMKEY_LOCALPATH  = "xMsgPrm_LocalPath";  //!< Str: local path
    public const string PGC_MSGPRMKEY_REMOTEPATH = "xMsgPrm_RemotePath"; //!< Str: remote path

    public const string PGC_MSGPRMKEY_SKIPSRSP   = "xMsgPrm_SkipsRsp";   //!< Bool Str: skips response

    //!@}


    //! \name KeyMsgSec
    //! Message keys section
    //!@{

    //! Messages
    public const string PGC_MSGKEY_PEERADDRPORT = "xMsg_PeerAddrPort"; //!< in
    public const string PGC_MSGKEY_SRCTRCNUM    = "xMsg_SrcTrcNum";    //!< in
    public const string PGC_MSGKEY_DSTTRCNUM    = "xMsg_DstTrcNum";    //!< in
    public const string PGC_MSGKEY_SRCUNAME     = "xMsg_SrcUName";     //!< in
    public const string PGC_MSGKEY_DSTUNAME     = "xMsg_DstUName";     //!< in
    public const string PGC_MSGKEY_USERNAME     = "xMsg_UserName";     //!< in
    public const string PGC_MSGKEY_PASSWORD     = "*xMsg_Password";    //!< in
    public const string PGC_MSGKEY_TOKEN        = "xMsg_Token";        //!< in
    public const string PGC_MSGKEY_METHOD       = "xMsg_Method";       //!< in

    public const string PGC_MSGKEY_ACTION         = "xMsg_Action";     //!< in
    public const string PGC_MSGVAL_ACTION_SEND    = "xMsgAct_Send";    //!< Action: Send    (client async)
    public const string PGC_MSGVAL_ACTION_REQUEST = "xMsgAct_Request"; //!< Action: Request (client sync/async)
    public const string PGC_MSGVAL_ACTION_REPLY   = "xMsgAct_Reply";   //!< Action: Reply   (server async)
    public const string PGC_MSGVAL_ACTION_NOTIFY  = "xMsgAct_Notify";  //!< Action: Notify  (server async)
    public const string PGC_MSGVAL_ACTION_CONFIRM = "xMsgAct_Confirm"; //!< Action: Confirm (client async)

    //!@}


    //! \name KeyTimeSec
    //! Stamp keys section
    //!@{

    //! Timestamps
    public const string PGC_TSKEY_STEP = "xTS_Step";  //!< Timestamp Step
    public const string PGC_TSPFX_NAME = "xTS_Name_"; //!< Timestamp Prefix: Name
    public const string PGC_TSPFX_IN   = "xTS_In_";   //!< Timestamp Prefix: In
    public const string PGC_TSPFX_OUT  = "xTS_Out_";  //!< Timestamp Prefix: Out

    //!@}


    //! \name KeyParamSec
    //! Parameter keys section
    //!@{

    //! Method Params
    public const string PGC_PRMKEY_ERRCODE = "xPrm_ErrCode"; //!< out

    //!@}

    ////////////////////////////////////////////////////////////////////////////////

    //! \name KeyC9Sec
    //! C9PG keys section
    //!@{

    //! C9PG: General
    public const string PGC_GWVAL_TRUE  = "Y"; //!< True, Yes
    public const string PGC_GWVAL_FALSE = "N"; //!< False, No
    public const string PGC_GWVAL_YES   = "Y"; //!< Yes
    public const string PGC_GWVAL_NO    = "N"; //!< No

    //! C9PG: Trace
    public const string PGC_GWKEY_SRCTRACENUM = "SourceTraceNum"; //!< Str: Source Trace Number; from POS to GW
    public const string PGC_GWKEY_GTRC        = "GTRC";           //!< Str: Gateway Trace; from GW to POS
    public const string PGC_GWKEY_GTRCLIST    = "GTRCList";       //!< StrList: Gateway Trace Number List; from POS/GW to GW/POS

    //! C9PG: Account
    public const string PGC_GWKEY_GMID = "GMID"; //!< Str: Gateway Merchant ID; from POS to GW
    public const string PGC_GWKEY_GTID = "GTID"; //!< Str: Gateway Device ID; from POS to GW
    public const string PGC_GWKEY_GMPW = "GMPW"; //!< Str: Gateway Merchant Password; from POS to GW

    //! C9PG: Option
    public const string PGC_GWKEY_ALLOWS_PARTIALAUTH = "AllowsPartialAuth"; //!< Bool: Allows Partial Auth; Def: N; from POS to GW
    public const string PGC_GWKEY_NEEDS_UI           = "NeedSwipeCard";     //!< Bool: Needs the UI popup (NeedSwipCard); from POS to GW

    //! C9PG: Action
    public const string PGC_GWKEY_TXNACT            = "TransType"; //!< Str: Transaction Action; Val: Auth, Sale, etc; from POS to GW
    public const string PGC_GWVAL_TXNACT_SALE       = "Sale";      //!< Transaction Action: Sale
    public const string PGC_GWVAL_TXNACT_AUTH       = "Auth";      //!< Transaction Action: Auth
    public const string PGC_GWVAL_TXNACT_INQUIRY    = "Inquiry";   //!< Transaction Action: Inquiry
    public const string PGC_GWVAL_TXNACT_REFUND     = "Refund";    //!< Transaction Action: Refund
    public const string PGC_GWVAL_TXNACT_FINALIZE   = "Finalize";  //!< Transaction Action: Finalize
    public const string PGC_GWVAL_TXNACT_ADDTIP     = "AddTip";    //!< Transaction Action: AddTip
    public const string PGC_GWVAL_TXNACT_MODIFY     = "Modify";    //!< Transaction Action: Modify
    public const string PGC_GWVAL_TXNACT_VOID       = "Void";      //!< Transaction Action: Void
    public const string PGC_GWVAL_TXNACT_BATCH      = "Batch";     //!< Transaction Action: Batch
    public const string PGC_GWVAL_TXNACT_ADDVALUE   = "AddValue";  //!< Transaction Action: AddValue
    public const string PGC_GWVAL_TXNACT_ACTIVATE   = "Active";    //!< Transaction Action: Activate
    public const string PGC_GWVAL_TXNACT_DEACTIVATE = "Deactive";  //!< Transaction Action: Deactivate
    public const string PGC_GWVAL_TXNACT_SELBATCH   = "SelBatch";  //!< Transaction Action: Selective Batch

    public const string PGC_GWKEY_ISOFFLINE = "IsOffline"; //!< Bool: Is Offline; Def: N; from POS to GW
    public const string PGC_GWKEY_AUTHCODE  = "AuthCode";  //!< Str: Authorization Code; = "from POS to GW"; for offline transactions, = "from GW to POS"; for regular transactions

    //! C9PG: Amount
    public const string PGC_GWKEY_MAINAMT      = "MainAmt";        //!< Amt: Main Amount; from POS to GW
    public const string PGC_GWKEY_TIPAMT       = "TipAmt";         //!< Amt: Tip Amount; from POS to GW
    public const string PGC_GWKEY_INCTAXAMT    = "IncTaxAmt";      //!< Amt: Included Tax Amount; from POS to GW
    public const string PGC_GWKEY_INCCASHBKAMT = "IncCashBackAmt"; //!< Amt: Included Cash Back Amount; from POS to GW

    //! C9PG: Tax
    public const string PGC_GWKEY_TAXIND        = "TaxIndicator"; //!< Str: Tax Indicator; Def: Prvded; from POS to GW
    public const string PGC_GWVAL_TAXIND_PRVDED = "Prvded";       //!< Tax Indicator: Provided
    public const string PGC_GWVAL_TAXIND_NTPRVD = "Ntprvd";       //!< Tax Indicator: Not Provided
    public const string PGC_GWVAL_TAXIND_NONTAX = "NonTax";       //!< Tax Indicator: Non-taxable

    //! C9PG: Info
    public const string PGC_GWKEY_INVOICENUM = "InvoiceNum"; //!< Str: Invoice Number; from POS to GW
    public const string PGC_GWKEY_ORDERNUM   = "OrderNum";   //!< Str: Order Number; from POS to GW
    public const string PGC_GWKEY_VOUCHERNUM = "VoucherNum"; //!< Str: Voucher Number; from POS to GW
    public const string PGC_GWKEY_ADDLINFO   = "AdditionalInfo"; //!< Str: Additional Info; key value pairs; from POS to GW

    //! C9PG: Card
    public const string PGC_GWKEY_ACCTNUM    = "*AccountNum"; //!< Str: Account Number; sensitive data; from POS to GW only if POS controls the enry
    public const string PGC_GWKEY_EXPDATE    = "ExpDate";    //!< Str: Expiration Date; Fmt: MMYY or MM/YY; from POS to GW only if POS controls the entry
    //public const string PGC_GWKEY_EXPYEAR  = "ExpYear";    //!< TBA; Str: Expiration Year
    //public const string PGC_GWKEY_EXPMONTH = "ExpMonth";   //!< TBA; Str: Expiration Month
    public const string PGC_GWKEY_CVVNUM     = "*CVVNum";     //!< Str: CVV, CVC Number; sensitive data; from POS to GW only if POS controls the entry
    //public const string PGC_GWKEY_TRACK    = "*Track";      //!< TBA; Str: MSR Track Data; sensitive data
    public const string PGC_GWKEY_TRACK1     = "*Track1";     //!< Str: MSR Track1; sensitive data; from POS to GW only if POS controls the entry
    public const string PGC_GWKEY_TRACK2     = "*Track2";     //!< Str: MSR Track2; sensitive data; from POS to GW only if POS controls the entry
    public const string PGC_GWKEY_PINDATA    = "*PinData";    //!< Str: PIN Data; sensitive data; from POS to GW only if POS controls the entry
    public const string PGC_GWKEY_ENCDATA    = "EncryptionData"; //!< Str: Encryption Data; required when account number is encrypted; from POS to GW only if POS controls the entry
    public const string PGC_GWKEY_ISCARDPRES = "CardPresent";    //!< Bool: Is Card Present; Def: Y; from POS to GW
    public const string PGC_GWKEY_ISPCARD    = "CommercialCard"; //!< Bool: Is Purchase / Commercial Card; Def: Y; from POS to GW if known, or from GW to POS using blanket support

    //! C9PG: Card Medium
    public const string PGC_GWKEY_MDM         = "Medium";  //!< Str: Medium; Val: Credit, Debit, Prepaid, EBT, etc; from POS to GW if known
    public const string PGC_GWVAL_MDM_CREDIT  = "Credit";  //!< Medium: Credit
    public const string PGC_GWVAL_MDM_DEBIT   = "Debit";   //!< Medium: Debit
    public const string PGC_GWVAL_MDM_GIFT    = "Gift";    //!< Medium: Gift
    public const string PGC_GWVAL_MDM_PREPAID = "Prepaid"; //!< Medium: Prepaid
    public const string PGC_GWVAL_MDM_EBTFOOD = "EBTFood"; //!< Medium: EBT Food
    public const string PGC_GWVAL_MDM_EBTCASH = "EBTCash"; //!< Medium: EBT Cash

    //! C9PG: Encryption
    public const string PGC_GWKEY_KEYID         = "KeyID";       //!< Str: Key ID; from POS to GW only if POS controls the entry
    public const string PGC_GWKEY_ENCBLK        = "EncrptBlock"; //!< Str: Encryption Block; encrypted track data or account number; from POS to GW only if POS controls the entry
    public const string PGC_GWKEY_ENCTGT        = "EncrptTrgt";  //!< Str: Encryption Target; Val: Track1, PAN; from POS to GW only if POS controls the entry
    public const string PGC_GWVAL_ENCTGT_TRACK1 = "Track1"; //!< Encryption Target: Track1; from POS to GW only if POS controls the entry
    public const string PGC_GWVAL_ENCTGT_TRACK2 = "Track2"; //!< Encryption Target: Track2; from POS to GW only if POS controls the entry
    public const string PGC_GWVAL_ENCTGT_PAN    = "PAN";    //!< Encryption Target: PAN; from POS to GW only if POS controls the entry

    //! C9PG: PIN
    public const string PGC_GWKEY_PINBLK = "*PinBlock"; //!< Str: PIN Block; sensitive data; from POS to GW only if POS controls the entry
    public const string PGC_GWKEY_KSN    = "KSN";      //!< Str: KSN; from POS to GW only if POS controls the entry

    //! C9PG: Result
    public const string PGC_GWKEY_STATUS             = "Status";      //!< Str: Status; from GW to POS
    public const string PGC_GWVAL_STATUS_OK          = "success";     //!< Status: OK
    public const string PGC_GWVAL_STATUS_FAILED      = "fail";        //!< Status: Failed
    public const string PGC_GWVAL_STATUS_TIMEDOUT    = "timeout";     //!< Status: Timed Out
    public const string PGC_GWVAL_STATUS_CANCELED    = "cancel";      //!< Status: Canceled
    public const string PGC_GWVAL_STATUS_DATAINVALID = "invalidData"; //!< Status: Data Invalid

    public const string PGC_GWKEY_RSPCODE = "ResponseCode"; //!< Str: Response Code (processor specific); from GW to POS
    public const string PGC_GWKEY_RSPTEXT = "ResponseText"; //!< Str: Response Text (processor specific); from GW to POS
    public const string PGC_GWKEY_ERRCODE = "ErrorCode";    //!< Str: Error Code (processor unified); from GW to POS
    public const string PGC_GWKEY_ERRTEXT = "ErrorText";    //!< Str: Error Text (processor unified); from GW to POS

    //public const string PGC_GWKEY_AUTHCODE = "AuthCode";      //!< Str: Authorization Code; = "from POS to GW"; for offline transactions, = "from GW to POS"; for regular transactions
    public const string PGC_GWKEY_AVSRSTCODE = "AVSResultCode"; //!< Str: AVS Result Code; from GW to POS
    public const string PGC_GWKEY_CVVRSTCODE = "CVVResultCode"; //!< Str: CVV Result Code; from GW to POS

    public const string PGC_GWKEY_AUTHAMT = "AuthAmt";    //!< Amt: Authorized Amount; from GW to POS
    public const string PGC_GWKEY_BALAMT  = "BalanceAmt"; //!< Amt: Balance Amount; from GW to POS

    //public const string PGC_GWKEY_MDM     = "Medium";     //!< Str: Medium; Val: Credit Card, Debit Card, Gift Card, EBT, etc; from GW to POS
    //public const string PGC_GWKEY_ACCTNUM = "AccountNum"; //!< Str: Account Number; from GW to POS

    //! C9PG: Card Brand
    public const string PGC_GWKEY_BRAND              = "Brand";        //!< Str: Brand; Val: Visa, Master, Discover, Amex, JCB, etc; from GW to POS
    public const string PGC_GWVAL_BRAND_VISA         = "Visa";         //!< Brand: Visa
    public const string PGC_GWVAL_BRAND_MASTERCARD   = "Master";       //!< Brand: MasterCard
    public const string PGC_GWVAL_BRAND_DISCOVER     = "Discover";     //!< Brand: Discover
    public const string PGC_GWVAL_BRAND_AMEX         = "Amex";         //!< Brand: American Express
    public const string PGC_GWVAL_BRAND_JCB          = "JCB";          //!< Brand: JCB
    public const string PGC_GWVAL_BRAND_CARTEBLANCHE = "CarteBlanche"; //!< Brand: Carte Blanche
    public const string PGC_GWVAL_BRAND_DINERSCLUB   = "DinersClub";   //!< Brand: Diners Club
    public const string PGC_GWVAL_BRAND_ENROUTE      = "EnRoute";      //!< Brand: EnRoute
    public const string PGC_GWVAL_BRAND_JAL          = "JAL";          //!< Brand: JAL
    public const string PGC_GWVAL_BRAND_DISNEYHOTEL  = "DisneyHotel";  //!< Brand: Disney Hotel
    public const string PGC_GWVAL_BRAND_DISNEYCARD   = "DisneyCard";   //!< Brand: Disney Card

    //! C9PG: Account Entry
    public const string PGC_GWKEY_ENTRMODE        = "EntryMode"; //!< Str: Entry Mode; from POS to GW only if POS controls the entry
    public const string PGC_GWVAL_ENTRMODE_MANUAL = "Manual";    //!< Entry Mode: Manual
    public const string PGC_GWVAL_ENTRMODE_SWIPE  = "Swipe";     //!< Entry Mode: Swipe

    //! C9PG: Tokenization
    public const string PGC_GWKEY_ACCTTOK         = "CardToken";        //!< Str: Account Token; representing an account number + exp date; from GW to POS on initial transaction response and from POS to GW on subsequent requests
    public const string PGC_GWKEY_REQACCTTOK      = "RequestCardToken"; //!< Bln: Request for Account Token; from POS to GW

    //! C9PG: Compatibility
    //! TXNACT (Transaction Action) = TransType
    //public const string PGC_GWKEY_TRANSTYPE          = "TransType"; // Str: Transaction Type; Val: Auth, Sale, etc
    //public const string PGC_GWVAL_TRANSTYPE_SALE     = "Sale";      // Transaction Type: Sale
    //public const string PGC_GWVAL_TRANSTYPE_AUTH     = "Auth";      // Transaction Type: Auth
    //public const string PGC_GWVAL_TRANSTYPE_INQUIRY  = "Inquiry";   // Transaction Type: Inquiry
    //public const string PGC_GWVAL_TRANSTYPE_REFUND   = "Refund";    // Transaction Type: Refund
    //public const string PGC_GWVAL_TRANSTYPE_FINALIZE = "Finalize";  // Transaction Type: Finalize
    //public const string PGC_GWVAL_TRANSTYPE_ADDTIP   = "AddTip";    // Transaction Type: AddTip
    //public const string PGC_GWVAL_TRANSTYPE_MODIFY   = "Modify";    // Transaction Type: Modify
    //public const string PGC_GWVAL_TRANSTYPE_VOID     = "Void";      // Transaction Type: Void
    //public const string PGC_GWVAL_TRANSTYPE_BATCH    = "Batch";     // Transaction Type: Batch
    //public const string PGC_GWVAL_TRANSTYPE_ADDVALUE = "AddValue";  // Transaction Type: AddValue
    //public const string PGC_GWVAL_TRANSTYPE_ACTIVE   = "Active";    // Transaction Type: Activate
    //public const string PGC_GWVAL_TRANSTYPE_DEACTIVE = "Deactive";  // Transaction Type: Deactivate

    //public const string PGC_GWKEY_ENTMODE        = "EntryMode"; //!< Str: Entry Mode; from POS to GW only if POS controls the entry
    //public const string PGC_GWVAL_ENTMODE_MANUAL = "Manual";    //!< Entry Mode: Manual

    //!@}


    //! \name KeyNumSec
    //! Number service keys section
    //!@{

    public const string PGC_NUMKEY_METHOD               = "Method";        //!< Str: Method
    public const string PGC_NUMVAL_METHOD_REGISTER      = "Register";      //!< Method: Register
    public const string PGC_NUMVAL_METHOD_UNREGISTER    = "Unregister";    //!< Method: Unregister
    public const string PGC_NUMVAL_METHOD_GETNEXT       = "GetNext";       //!< Method: GetNext

    public const string PGC_NUMKEY_ERRCODE              = "ErrCode";       //!< Int: Error Code

    public const string PGC_NUMKEY_TAG                  = "Tag";           //!< Str: Tag
    public const string PGC_NUMKEY_PASSWORD             = "Password";      //!< Str: Password
    public const string PGC_NUMKEY_COUNT                = "Count";         //!< Int: Count
    public const string PGC_NUMKEY_NUMLIST              = "NumList";       //!< Str: Number list; separated by commas if counter > 1

    public const string PGC_NUMKEY_MIN                  = "Min";           //!< Int: Minimum
    public const string PGC_NUMKEY_MAX                  = "Max";           //!< Int: Maximum
    public const string PGC_NUMKEY_RANDOMIZE            = "Randomize";     //!< Bln: Randomize
    public const string PGC_NUMKEY_ADDMOD10DIGIT        = "AddMod10Digit"; //!< Bln: Add Mod10 Digit
    public const string PGC_NUMKEY_PREFIX               = "Prefix";        //!< Str: Prefix
    public const string PGC_NUMKEY_SUFFIX               = "Suffix";        //!< Str: Suffix

    //!@}

    ////////////////////////////////////////////////////////////////////////////////

    //! Notes
    //! Pointer style JSON keys: /RootObj/SubObj/ArrIdx/Key
    //! Pointer style property keys: RootObj.SubObj-ArrIdx.Key


}
